
import os
import sys
import shutil
from pathlib import Path
from subprocess import run, PIPE


def init_checks(config_file, cluster_context, execute_helm_init=False):
    """
    Perform general checks,
    Check compatible python version,
    executable 'kubectl' command
    export current config_file as environment variable (KUBECONFIG)
    setup cluster context
    existence of 'config_local' file
    executable 'helm' command

    :param config_file: String
    :param cluster_context: String
    :param execute_helm_init: Boolean
    :return:
    """
    # Make sure python 3.5 and above is used
    if sys.version_info[0] < 3 or sys.version_info[1] < 5:
        print("Python 3.5 is a minimal requirement")
        exit(1)

    # Make sure config_file exists
    if not Path(config_file).exists():
        print("Could not locate {}\nMake sure to create it".format(config_file))
        exit(1)

    # Make sure kubectl command exists
    if shutil.which("kubectl") is None:
        print("Could not execute 'kubectl' command\nMake sure 'kubectl' installed")
        exit(1)

    # Export config_file since the user may exported other file on same terminal process
    os.environ["KUBECONFIG"] = str(config_file)

    print("Config file used: {}\n"
          "Context used: {}".format(config_file, cluster_context))
    use_context_output = run(["kubectl", "config", "--kubeconfig", config_file,
                              "use-context", cluster_context], stdout=PIPE, stderr=PIPE)
    # In case returncode is not 0
    if use_context_output.returncode:
        print(use_context_output.stderr.decode('utf-8') +
              "Make sure your cluster is up and running, "
              "and '{}' file contains correct values".format(config_file))
        exit(1)

    # .run return CompletedProcess with returncode and stdout and stderr values as bytes object, using UTF-8 decode
    print("Retrieving cluster information")
    cluster_info_output = run(["kubectl", "cluster-info"], stdout=PIPE, stderr=PIPE)

    # In case returncode is not 0
    if cluster_info_output.returncode:
        print(cluster_info_output.stderr.decode('utf-8') +
              "Make sure your cluster is up and running, "
              "and '{}' file contains correct values".format(config_file))
        exit(1)
    # else:
    #     print(cluster_info_output.stdout.decode('utf-
    # Make sure helm command exists
    if shutil.which("helm") is None:
        print("Could not execute 'helm' command,\n"
              "Make sure 'helm' installed: https://docs.helm.sh/using_helm/#installing-helm")
        exit(1)

    if execute_helm_init:
        print("Executing 'helm init' command")
        helm_init_output = run(["helm", "init"], stdout=PIPE, stderr=PIPE)
        # In case returncode is not 0
        if helm_init_output.returncode:
            print(helm_init_output.stderr.decode('utf-8'))
            exit(1)
    # else:
    #     print(helm_init_output.stdout.decode('utf-8'))




import subprocess


def get_cluster_info():
    """
    Get cluster information

    :return: return value from command execution as string
    """
    cluster_info_completed_process = subprocess.run(["kubectl", "cluster-info"],
                                                    stdout=subprocess.PIPE,
                                                    stderr=subprocess.PIPE)

    if cluster_info_completed_process.returncode == 0:
        return cluster_info_completed_process.stdout.decode('utf-8').strip()
    else:
        return cluster_info_completed_process.stderr.decode('utf-8').strip()


def create_registry_secret(secret_name, docker_registry, docker_username, docker_password, namespace):
    """
    Create a new docker-registry secret

    :param secret_name: String
    :param docker_registry: String
    :param docker_username: String
    :param docker_password: String
    :param namespace: String
    :return: return code and value from command execution as dictionary
    """
    add_registry_completed = subprocess.run(["kubectl", "create", "secret", "docker-registry", secret_name,
                                             "--docker-server=%s" % docker_registry,
                                             "--docker-username=%s" % docker_username,
                                             "--docker-password=%s" % docker_password,
                                             "--namespace=%s" % namespace],
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)
    if add_registry_completed.returncode == 0:
        value = add_registry_completed.stdout.decode('utf-8').strip()
    else:
        value = add_registry_completed.stderr.decode('utf-8').strip()

    return {'returncode': add_registry_completed.returncode, 'value': value}




import urwid
from subprocess import run, PIPE


class DeleteChartsMenu:

    def __init__(self, return_func, helm_charts_installed_dict):

        self.selection_ch_box = [urwid.CheckBox("{:30}{}".format(repo['chart_name'], repo['name_space']))
                                 for repo in helm_charts_installed_dict]

        self.text_deletion_result = urwid.Text(u"")

        blank = urwid.Divider()
        listbox_content = [
            blank,
            urwid.WidgetWrap(urwid.Divider("=", 1)),
            urwid.Padding(urwid.Text(u"Helm Charts Removal"), left=2, right=2, min_width=20),
            urwid.WidgetWrap(urwid.Divider("*", 0, 1)),
            urwid.Padding(urwid.Text(u"Use space/enter to mark helm chart to delete"), left=2, right=2, min_width=20),
            blank,

            # Display helm charts block
            urwid.Padding(
                urwid.Pile([
                    urwid.Text("    {:30}{}".format("NAME", "NAMESPACE")),
                    urwid.Pile(
                        [urwid.AttrWrap(chart, 'buttn', 'buttnf') for chart in self.selection_ch_box],
                    ),

                ]), left=2, right=2, min_width=10),

            blank,
            urwid.Padding(
                urwid.GridFlow(
                    [urwid.AttrWrap(urwid.Button("Cancel", on_press=self.on_cancel), 'buttn', 'buttnf'),
                     blank,
                     urwid.AttrWrap(urwid.Button("Delete Selected Charts", on_press=self.on_delete), 'buttn', 'buttnf')
                     ],
                    20, 1, 8, 'left'),
                left=2, right=2, min_width=20, align='left'),
            blank,
            urwid.Padding(self.text_deletion_result, left=2, right=2, min_width=20),
        ]

        listbox = urwid.ListBox(urwid.SimpleListWalker(listbox_content))

        self.return_func = return_func

        self.main_window = urwid.LineBox(urwid.AttrWrap(listbox, 'body'))

    def on_cancel(self, w):
        self.return_func()

    def on_delete(self, w):
        charts_to_delete = []
        for chart in self.selection_ch_box:
            if chart.get_state():
                # split() without arguments splits on whitespace, get first cell of split
                charts_to_delete.append(chart.label.split()[0])
        # If the array remained empty
        if not charts_to_delete:
            self.text_deletion_result.set_text(u"At least one chart must be selected")
        else:
            result = delete_helm_installations(charts_to_delete)
            self.return_func(True)
        # self.text_installation_result.set_text(u'%s' % result)


def delete_helm_installations(charts_array):
    """
    Execute 'helm delete' command on input values
    charts_array is an array of strings as:
    ['ingress-traefik', 'kubernetes-dashboard', ...]

    :param charts_array: array of strings
    :return: return code and value from execution command as dict
    """
    status = 0
    value = 'no errors found'
    for installation in charts_array:
        completed_process_object = run(["helm", "delete", "--purge", installation], stdout=PIPE, stderr=PIPE)
        # In case of a non 0 return code, update return from last iteration
        if completed_process_object.returncode != 0:
            status = completed_process_object.returncode
            value = completed_process_object.stderr.decode('utf-8') + " *** Additional errors may occurred"
    return {'status': status, 'value': value}




import re


def remove_ansi_color_from_string(input_string):
    ansi_escape = re.compile(r'(\x9B|\x1B\[)[[0-?]*[ -/]*[@-~]')
    return ansi_escape.sub('', input_string)


main_palette = [
    ('body', 'black', 'light gray', 'standout'),
    ('reverse', 'light gray', 'black'),
    ('header', 'white', 'dark red', 'bold'),
    ('footer', 'white', 'dark green', 'bold'),
    ('important', 'dark blue', 'light gray', ('standout', 'underline')),
    ('editfc', 'white', 'dark blue', 'bold'),
    ('editbx', 'light gray', 'dark blue'),
    ('editcp', 'black', 'light gray', 'standout'),
    ('bright', 'dark gray', 'light gray', ('bold', 'standout')),
    ('buttn', 'black', 'light blue'),
    ('buttnf', 'white', 'dark blue', 'bold'),
    ('errors', 'black', 'dark red'),
]




import argparse
import urwid
import urwid.raw_display
import urwid.web_display
from pathlib import Path
from helm_charts.helm_functions import *
from helm_charts.init_checks import init_checks
from helm_charts.kubectl_functions import *
from helm_charts.helper import *
from helm_charts.menu_install_charts import InstallChartsMenu
from helm_charts.menu_delete_charts import DeleteChartsMenu
from helm_charts.menu_delete_repos import DeleteReposMenu
from helm_charts.menu_install_repo import AddRepoMenu
# from helm_functions import *
# from init_checks import init_checks
# from kubectl_functions import *
# from helper import *
# from menu_install_charts import InstallChartsMenu
# from menu_delete_charts import DeleteChartsMenu
# from menu_delete_repos import DeleteReposMenu
# from menu_install_repo import AddRepoMenu

config_file = str(Path.home()) + "/.kube/config"
cluster_context = "docker-for-desktop"


class MainView(urwid.WidgetPlaceholder):

    def __init__(self):
        text_header = (u"Helm charts installer -  "
                       u"UP / DOWN / PAGE UP / PAGE DOWN scroll.  ctrl+e exits.")
        text_footer = remove_ansi_color_from_string(get_cluster_info().split("\n")[0])
        text_intro = [('important', u"Helm charts installer "),
                      u"installs kubernetes helm charts on a configured cluster, "
                      u"by default its intended to work on a cluster on the local host. "
                      u"For more details execute 'helm_charts -h'"]
        config_context_info = [u"Current config file:   ", ('important', config_file),
                               u"\nCurrent context used:  ", ('important', cluster_context)]

        self.helm_repo_installed_repositories_text = urwid.Text(
            identify_installed_helm_repos(return_only_decoded_string=True).replace("\t", "    "))

        self.text_helm_charts_installation_result = urwid.Text(u"", align='left')

        self.helm_repo_change_result = urwid.Text('')

        self.text_helm_charts_installed = urwid.Text(
            identify_installed_helm_charts(return_only_decoded_string=True).replace("\t", "    ")
        )

        blank = urwid.Divider()
        equal_divider = urwid.WidgetWrap(urwid.Divider("=", 1))
        asterisk_divider = urwid.WidgetWrap(urwid.Divider("*", 0, 1))
        listbox_content = [
            # Show into text
            blank,
            urwid.Padding(urwid.Text(text_intro), left=2, right=2, min_width=20),

            # Show current config file, and context used
            blank,
            urwid.Padding(urwid.Text(config_context_info), left=2, right=2, min_width=20),

            # Helm charts display
            equal_divider,
            urwid.Padding(urwid.Text([u"Installed ", ('important', u"Helm charts:")]), left=2, right=2, min_width=20),
            asterisk_divider,
            urwid.Padding(self.text_helm_charts_installed, left=2, right=2, min_width=20),
            blank,
            urwid.Padding(urwid.GridFlow(
                [urwid.AttrWrap(urwid.Button("Install Charts", self.on_chart_install_open), 'buttn', 'buttnf')],
                18, 3, 1, 'left'),
                left=2, right=2, min_width=13),
            blank,
            urwid.Padding(urwid.GridFlow(
                [urwid.AttrWrap(urwid.Button("Delete Charts", self.on_chart_delete_open), 'buttn', 'buttnf')],
                18, 3, 1, 'left'),
                left=2, right=2, min_width=13),
            blank,
            # Print results of charts installations
            urwid.Padding(self.text_helm_charts_installation_result, left=2, right=2, min_width=20),

            # Helm repositories display
            equal_divider,
            urwid.Padding(
                urwid.Text([u"Installed ", ('important', u"Helm repositories:")]), left=2, right=2, min_width=20),
            asterisk_divider,
            urwid.Padding(self.helm_repo_installed_repositories_text, left=2, right=2, min_width=20),
            blank,
            urwid.Padding(
                urwid.AttrWrap(
                    urwid.Button("Add repository", self.on_repo_add_menu_open), 'buttn', 'buttnf'),
                left=2, right=2, width=25),
            blank,
            urwid.Padding(
                urwid.AttrWrap(
                    urwid.Button("Delete repositories", self.on_repo_delete_menu_open), 'buttn', 'buttnf'),
                left=2, right=2, width=25),
            blank,
            # Print result from helm repo add
            urwid.Padding(self.helm_repo_change_result, left=2, right=2, min_width=20),
        ]

        header = urwid.AttrWrap(urwid.Text(text_header), 'header')
        footer = urwid.AttrWrap(urwid.Text(text_footer), 'footer')
        listbox = urwid.ListBox(urwid.SimpleListWalker(listbox_content))

        # Frame widget is used to keep the instructions at the top of the screen
        self.frame = urwid.Frame(urwid.AttrWrap(listbox, 'body'), header=header, footer=footer)

        urwid.WidgetPlaceholder.__init__(self, self.frame)

    def button_press(self, button):
        self.frame.footer = urwid.AttrWrap(urwid.Text(
            [u"Pressed: ", button.get_label()]), 'header')

    def on_chart_install_open(self, w):
        """
        Execute once 'Install Charts' button pressed,
        init the 'InstallChartsMenu' class, and perform an overlay to that class

        :param w:
        :return:
        """
        install_charts_menu = InstallChartsMenu(self.on_charts_menu_close, supported_helm_deployments)
        self.original_widget = urwid.Overlay(install_charts_menu.main_window, self.original_widget,
                                             align='center', width=120, valign='middle', height=30)

    def on_chart_delete_open(self, w):
        """
        Execute once 'Delete Charts' button pressed,
        init the 'InstallChartsMenu' class, and perform an overlay to that class

        :param w:
        :return:
        """
        delete_charts_menu = DeleteChartsMenu(self.on_charts_menu_close, identify_installed_helm_charts())
        self.original_widget = urwid.Overlay(delete_charts_menu.main_window, self.original_widget,
                                             align='center', width=120, valign='middle', height=30)

    def on_charts_menu_close(self, refresh_installed_charts=False, returned_result=None):
        # refresh_installed_charts parameter is checked to determine if installed charts text needs update
        if refresh_installed_charts:
            self.text_helm_charts_installed.set_text(
                identify_installed_helm_charts(return_only_decoded_string=True).replace("\t", "    ")
            )
            # If this function returned with some result
            if returned_result is not None and 'status' in returned_result:
                # In case there where no errors print nothing
                if returned_result['status'] == 0:
                    self.text_helm_charts_installation_result.set_text(u"")
                else:
                    self.text_helm_charts_installation_result.set_text(("errors", returned_result['value']))
        self.original_widget = self.frame

    def on_repo_add_menu_open(self, w):
        """
        Execute once 'Add repository' button pressed,
        init the 'InstallChartsMenu' class, and perform an overlay to that class

        :param w:
        :return:
        """
        add_repo_menu = AddRepoMenu(self.on_repos_menu_closed)
        self.original_widget = urwid.Overlay(add_repo_menu.main_window, self.original_widget,
                                             align='center', width=120, valign='middle', height=30)

    def on_repo_delete_menu_open(self, button):
        delete_repos_menus = DeleteReposMenu(self.on_repos_menu_closed, identify_installed_helm_repos())
        self.original_widget = urwid.Overlay(delete_repos_menus.main_window, self.original_widget,
                                             align='center', width=120, valign='middle', height=30)

    def on_repos_menu_closed(self, refresh_installed_repos=False, returned_result=None):
        if refresh_installed_repos:
            # Reset the installed repos text area
            self.helm_repo_installed_repositories_text.set_text(
                identify_installed_helm_repos(return_only_decoded_string=True).replace("\t", "    ")
            )
            if returned_result is not None and 'status' in returned_result:
                # In case there where no errors print nothing
                if returned_result['status'] == 0:
                    self.helm_repo_change_result.set_text(u"")
                else:
                    self.helm_repo_change_result.set_text(("errors", returned_result['value']))
        self.original_widget = self.frame


class HelmInstaller:

    def __init__(self):

        self.frame = MainView()

    def main(self):
        if urwid.web_display.is_web_request():
            screen = urwid.web_display.Screen()
        else:
            screen = urwid.raw_display.Screen()
        urwid.MainLoop(self.frame, main_palette, screen, unhandled_input=self.unhandled).run()

    def unhandled(self, key):
        if key == 'ctrl e':
            raise urwid.ExitMainLoop()


def main():
    urwid.web_display.set_preferences("Urwid Tour")
    # try to handle short web requests quickly
    if urwid.web_display.handle_short_request():
        return

    global config_file
    global cluster_context

    parser = argparse.ArgumentParser(description='Python Helm Charts Installer')
    parser.add_argument('--config-file', help='Path to kubernetes config file, '
                                              'Default is `{}`'.format(config_file), required=False)
    parser.add_argument('--use-context', help='Cluster context name to use, '
                                              'Default is `{}`'.format(cluster_context), required=False)
    parser.add_argument('--helm-init', action='store_true', help='Perform "helm init" on cluster', required=False)

    args = vars(parser.parse_args())

    if args['config_file'] is not None:
        config_file = args['config_file']

    if args['use_context'] is not None:
        cluster_context = args['use_context']

    print("Starting application, please wait...")
    init_checks(config_file, cluster_context, args['helm_init'])

    helm_installer = HelmInstaller()
    helm_installer.main()


if __name__ == '__main__':
    main()




import urwid
from subprocess import run, PIPE


class AddRepoMenu:

    def __init__(self, return_func):

        self.helm_repo_name_edit_box = urwid.Edit(('editcp', u"Helm repo name "), "set name", align='left', )
        self.helm_repo_url_edit_box = urwid.Edit(('editcp', u"Helm repo URL  "), "set url", align='left', )
        self.add_repo_result = urwid.Text(u"")

        blank = urwid.Divider()
        listbox_content = [
            blank,
            urwid.WidgetWrap(urwid.Divider("=", 1)),
            urwid.Padding(urwid.Text(u"Add Helm Repository"), left=2, right=2, min_width=20),
            urwid.WidgetWrap(urwid.Divider("*", 0, 1)),
            blank,

            blank,
            urwid.Padding(urwid.AttrWrap(self.helm_repo_name_edit_box, 'editbx', 'editfc'), left=2, right=2),
            urwid.Padding(urwid.AttrWrap(self.helm_repo_url_edit_box, 'editbx', 'editfc'), left=2, right=2),
            blank,
            urwid.Padding(
                urwid.GridFlow(
                    [urwid.AttrWrap(urwid.Button("Cancel", on_press=self.on_cancel), 'buttn', 'buttnf'),
                     blank,
                     urwid.AttrWrap(urwid.Button("Add repository", on_press=self.on_add_repo), 'buttn', 'buttnf')
                     ],
                    20, 1, 8, 'left'),
                left=2, right=2, min_width=20, align='left'),

            blank,
            urwid.Padding(self.add_repo_result, left=2, right=2, min_width=20),
        ]

        listbox = urwid.ListBox(urwid.SimpleListWalker(listbox_content))

        self.return_func = return_func
        self.main_window = urwid.LineBox(urwid.AttrWrap(listbox, 'body'))

    def on_cancel(self, w):
        self.return_func()

    def on_add_repo(self, w):
        """
        Execute once 'Add repository' button pressed,
        init the 'add_helm_repo' function,
        update the 'text_helm_repo_installed_repositories' urwid.TEXT object
        and update 'helm_repo_change_result' urwid.TEXT object

        :return:
        """

        # Execute the 'add repo' function with relevant values, and get the 'value' return from function
        add_helm_repo_output = add_helm_repo(self.helm_repo_name_edit_box.edit_text,
                                             self.helm_repo_url_edit_box.edit_text)
        if add_helm_repo_output['status'] != 0:
            self.add_repo_result.set_text(add_helm_repo_output['value'])
        else:
            return self.return_func(refresh_installed_repos=True, returned_result=add_helm_repo_output['value'])


def add_helm_repo(repo_name, repo_url):
    """
    Execute 'helm repo add' command on input values

    :param repo_name: name of repository as strings
    :param repo_url: url of repository as string
    :return: return code and value from execution command as dict
    """
    # execute and get CompletedProcess object
    completed_process_object = run(["helm", "repo", "add", repo_name, repo_url], stdout=PIPE, stderr=PIPE)
    if completed_process_object.returncode == 0:
        value = completed_process_object.stdout.decode('utf-8')
    else:
        value = completed_process_object.stderr.decode('utf-8')
    return {'status': completed_process_object.returncode, 'value': value}




import subprocess
import os

values_dir_path = os.path.dirname(os.path.realpath(__file__)) + '/values_files/'

supported_helm_deployments = [
    {'chart_name': 'ingress-traefik',
     'helm_repo_name': 'stable/traefik',
     'name_space': 'ingress-traefik',
     'values_file': values_dir_path + 'ingress-traefik.values.local.yml',
     'private_image': False},

    {'chart_name': 'kubernetes-dashboard',
     'helm_repo_name': 'stable/kubernetes-dashboard',
     'name_space': 'kube-system',
     'values_file': values_dir_path + 'kubernetes-dashboard.values.local.yml',
     'private_image': False},

    {'chart_name': 'jenkins',
     'helm_repo_name': 'stable/jenkins',
     'name_space': 'jenkins',
     'values_file': values_dir_path + 'jenkins.values.local.yml',
     'private_image': False},

    {'chart_name': 'nalkinscloud-nginx',
     'helm_repo_name': 'nalkinscloud/nalkinscloud-nginx',
     'name_space': 'nalkinscloud-nginx',
     'values_file': values_dir_path + 'nalkinscloud-nginx.values.local.yml',
     'private_image': False},

    {'chart_name': 'nalkinscloud-frontend',
     'helm_repo_name': 'nalkinscloud/nalkinscloud',
     'name_space': 'nalkinscloud-frontend',
     'values_file': values_dir_path + 'nalkinscloud.frontend.values.local.yml',
     'private_image': True},

    {'chart_name': 'nalkinscloud-api',
     'helm_repo_name': 'nalkinscloud/nalkinscloud',
     'name_space': 'nalkinscloud-api',
     'values_file': values_dir_path + 'nalkinscloud.api.values.local.yml',
     'private_image': True},
]


def identify_installed_helm_repos(return_only_decoded_string=False):
    """
    Function will perform a manipulation on a string output from the 'helm repo list' command
    If 'return_only_decoded_string' True, do not perform any string processing

    Returns an array of dicts with installed repos names and urls as strings
    as [{'repo_name': 'some_repo_name', 'repo_url': 'some_repo_url'}]

    'helm repo list' command return example:
    NAME        	URL
    stable      	https://kubernetes-charts.storage.googleapis.com
    local       	http://127.0.0.1:8879/charts
    nalkinscloud	https://arielevs.github.io/Kubernetes-Helm-Charts/

    by validating the first line, splitting by the tab delimiter,
    and checking that the first (0) value is 'NAME' and second (1) value is 'URL'
    an exception will be raised if the structure was change by HELM developers

    :param return_only_decoded_string: if True then return decoded (utf-8) "original" 'helm repo list' command output
    :return: array of dicts with repo installations
    """
    # Execute 'helm list' command, returned as CompletedProcess
    installed_repos_completed_process = subprocess.run(["helm", "repo", "list"],
                                                       stdout=subprocess.PIPE,
                                                       stderr=subprocess.PIPE)

    if return_only_decoded_string:
        if installed_repos_completed_process.returncode == 0:
            value = installed_repos_completed_process.stdout.decode('utf-8').strip()
        else:
            value = installed_repos_completed_process.stderr.decode('utf-8').strip()
        return value

    installed_repos = []

    # In case returncode is 0
    if not installed_repos_completed_process.returncode:
        # get stdout from installed_repos_completed_process, and decode for 'utf-8'
        # split stdout of installed_repos_completed_process by 'new line'
        installed_repos_stdout = installed_repos_completed_process.stdout.decode('utf-8').split("\n")

        # Perform validation on stdout of first (0) line
        first_line_stdout = installed_repos_stdout[0].split("\t")
        if first_line_stdout[0].strip() != 'NAME' or first_line_stdout[1].strip() != 'URL':
            raise Exception("'helm repo list' command output changed, "
                            "code change is needed to resolve this issue, "
                            "contact the developer.")

        # for every line in installed repos, excluding the headers line (NAME and URL)
        for line in installed_repos_stdout[1:]:
            # each stdout 'helm repo list' line composed by tabs delimiter, split it
            repo_details = line.split("\t")

            temp_dictionary = {}
            if repo_details[0] != "":
                # Add current line repo values to dict
                temp_dictionary.update({'repo_name': repo_details[0].strip()})
                temp_dictionary.update({'repo_url': repo_details[1].strip()})
                # Update final array with the temp array of dicts of current repo
                installed_repos.append(temp_dictionary)

    return installed_repos


def identify_installed_helm_charts(return_only_decoded_string=False):
    """
    Function will perform a manipulation on a string output from the 'helm list' command
    If 'return_only_decoded_string' True, do not perform any string processing

    Returns an array of dicts with installed chart names and namespaces as strings
    as [{'chart_name': 'some_chart_name', 'name_space': 'some_name_space'}]

    'helm list' command return example:
    NAME                	REVISION	UPDATED                 	STATUS  	CHART                     	NAMESPACE
    ingress-traefik     	2       	Thu Dec 27 19:45:01 2018	DEPLOYED	traefik-1.56.0            	ingress-traefik
    kubernetes-dashboard	11      	Sun Sep 16 11:21:24 2018	DEPLOYED	kubernetes-dashboard-0.7.3	kube-system

    by validating the first line, splitting by the tab delimiter,
    and checking that the first (0) value is 'NAME' and sixth (5) value is 'NAMESPACE'
    an exception will be raised if the structure was change by HELM developers

    :param return_only_decoded_string: if True then return decoded (utf-8) "original" 'helm list' command output
    :return: array of dicts with helm installations
    """
    # Execute 'helm list' command, returned as CompletedProcess
    installed_helm_completed_process = subprocess.run(["helm", "list"],
                                                      stdout=subprocess.PIPE,
                                                      stderr=subprocess.PIPE)

    if return_only_decoded_string:
        if installed_helm_completed_process.returncode == 0:
            value = installed_helm_completed_process.stdout.decode('utf-8').strip()
        else:
            value = installed_helm_completed_process.stderr.decode('utf-8').strip()
        return value

    installed_charts = []

    # In case returncode is 0
    if not installed_helm_completed_process.returncode:
        # get stdout from installed_helm_completed_process, and decode for 'utf-8'
        # split stdout of installed_helm_completed_process by 'new line'
        installed_helm_stdout = installed_helm_completed_process.stdout.decode('utf-8').split("\n")

        # Perform validation on stdout of first (0) line
        first_line_stdout = installed_helm_stdout[0].split("\t")
        if first_line_stdout[0].strip() != 'NAME' or first_line_stdout[5].strip() != 'NAMESPACE':
            raise Exception("'helm list' command output changed, "
                            "code change is needed to resolve this issue, "
                            "contact the developer.")

        # for every line in installed charts, excluding the headers line (Name, Revision, Updated etc...)
        for line in installed_helm_stdout[1:]:
            # each stdout 'helm list' line composed by tabs delimiter, split it
            chart_details = line.split("\t")

            temp_dictionary = {}
            if chart_details[0] != "":
                # Add current line chart values to dict
                temp_dictionary.update({'chart_name': chart_details[0].strip()})
                temp_dictionary.update({'name_space': chart_details[5].strip()})
                # Update final array with the temp array of dicts of current helm deployment
                installed_charts.append(temp_dictionary)

    return installed_charts


def identify_charts_in_repo(repository_name, return_only_decoded_string=False):
    """
    Execute 'helm repo add' command on input values

    :param repository_name: name of repository as strings
    :param return_only_decoded_string: if True then return decoded (utf-8) "original" 'helm search' command output
    :return: return code and value from execution command as dict
    """
    # execute and get CompletedProcess object
    completed_process_object = subprocess.run(["helm", "search", repository_name],
                                              stdout=subprocess.PIPE,
                                              stderr=subprocess.PIPE)
    return_code = completed_process_object.returncode
    if return_code != 0:
        return {'status': return_code,
                'value': completed_process_object.stderr.decode('utf-8').strip()}
    else:
        if return_only_decoded_string:
            return {'status': return_code,
                    'value': completed_process_object.stdout.decode('utf-8').strip()}

        # get stdout from completed_process_object, and decode for 'utf-8'
        # split stdout of completed_process_object by 'new line'
        search_repos_stdout = completed_process_object.stdout.decode('utf-8').split("\n")

        # Perform validation on stdout of first (0) line
        first_line_stdout = search_repos_stdout[0].split("\t")
        if first_line_stdout[0].strip() != 'NAME' or first_line_stdout[3].strip() != 'DESCRIPTION':
            raise Exception("'helm search' command output changed, "
                            "code change is needed to resolve this issue, "
                            "contact the developer.")

        charts_list = []
        # for every line in search repo output, excluding the headers line (NAME, CHART VERSION etc)
        for line in search_repos_stdout[1:]:
            # each stdout 'helm search' line composed by tabs delimiter, split it
            chart_details = line.split("\t")

            temp_dictionary = {}
            if chart_details[0] != "":
                # Add current line chart values to dict
                temp_dictionary.update({'chart_name': chart_details[0].strip()})
                temp_dictionary.update({'chart_version': chart_details[1].strip()})
                temp_dictionary.update({'app_version': chart_details[2].strip()})
                temp_dictionary.update({'description': chart_details[3].strip()})
                # Update final array with the temp array of dicts of current charts
                charts_list.append(temp_dictionary)

    return {'status': return_code, 'value': charts_list}




import urwid
from subprocess import run, PIPE


class InstallChartsMenu:

    def __init__(self, return_func, charts_dict_list):

        self.charts_dict_list = charts_dict_list

        self.docker_registry = ''
        self.docker_username = ''
        self.docker_password = ''

        # Define array of CheckBox, from values received from helm_charts_installed_dict
        self.selection_ch_box = [urwid.CheckBox("{:30}{}".format(chart['chart_name'], chart['name_space']))
                                 for chart in self.charts_dict_list]
        self.docker_registry_edit_box = urwid.Edit(
            ('editcp', u"Docker Registry "), self.docker_registry, align='left',
        )
        self.docker_username_edit_box = urwid.Edit(
            ('editcp', u"Docker Username "), self.docker_username, align='left',
        )
        self.docker_password_edit_box = urwid.Edit(
            ('editcp', u"Docker Password "), self.docker_password, align='left',
        )
        self.text_installation_result = urwid.Text(u"")

        blank = urwid.Divider()
        listbox_content = [
            blank,
            urwid.WidgetWrap(urwid.Divider("=", 1)),
            urwid.Padding(urwid.Text(u"Helm Charts Installation"), left=2, right=2, min_width=20),
            urwid.WidgetWrap(urwid.Divider("*", 0, 1)),
            urwid.Padding(urwid.Text(u"Use space/enter to mark helm chart to install"), left=2, right=2, min_width=20),
            blank,

            # Display helm charts block
            urwid.Padding(
                urwid.Pile([
                    urwid.Text("    {:30}{}".format("NAME", "NAMESPACE")),
                    urwid.Pile(
                        [urwid.AttrWrap(chart, 'buttn', 'buttnf') for chart in self.selection_ch_box],
                    ),

                ]), left=2, right=2, min_width=10),
            blank,
            urwid.Padding(
                urwid.Text(
                    [u"In case chart contains ", ('important', u"docker repository secrets"), ", set the below:"]
                ), left=2, right=2, min_width=20
            ),
            urwid.Padding(urwid.AttrWrap(self.docker_registry_edit_box, 'editbx', 'editfc'), left=2, right=2),
            urwid.Padding(urwid.AttrWrap(self.docker_username_edit_box, 'editbx', 'editfc'), left=2, right=2),
            urwid.Padding(urwid.AttrWrap(self.docker_password_edit_box, 'editbx', 'editfc'), left=2, right=2),

            blank,
            urwid.Padding(
                urwid.GridFlow(
                    [urwid.AttrWrap(urwid.Button("Cancel",
                                                 on_press=self.on_cancel), 'buttn', 'buttnf'),
                     blank,
                     urwid.AttrWrap(urwid.Button("Install Selected Charts",
                                                 on_press=self.on_install), 'buttn', 'buttnf')
                     ],
                    20, 1, 8, 'left'),
                left=2, right=2, min_width=20, align='left'),
            blank,

            urwid.Padding(self.text_installation_result, left=2, right=2, min_width=20),
        ]

        listbox = urwid.ListBox(urwid.SimpleListWalker(listbox_content))

        self.return_func = return_func

        self.main_window = urwid.LineBox(urwid.AttrWrap(listbox, 'body'))

    def on_cancel(self, w):
        self.return_func()

    def on_install(self, w):
        charts_to_install = []
        for index, chart in enumerate(self.selection_ch_box):
            if chart.get_state():
                # split() without arguments splits on whitespace, get first cell of split
                charts_to_install.append(chart.label.split()[0])

                # TODO un bind the dependence between charts_dict_list and selection_ch_box
                # Since selection_ch_box is built from charts_dict_list,
                # they share same values on same indexes
                if self.charts_dict_list[index]['private_image']:

                    self.docker_registry = self.docker_registry_edit_box.edit_text
                    self.docker_username = self.docker_username_edit_box.edit_text
                    self.docker_password = self.docker_password_edit_box.edit_text

                    if self.docker_registry == '' or self.docker_username == '' or self.docker_password == '':
                        self.text_installation_result.set_text(
                            u"One or more of the selected helm chart require docker-registry-secret, "
                            u"All fields must be filled"
                        )
                        return

        # If the array remained empty
        if not charts_to_install:
            self.text_installation_result.set_text(u"At least one chart must be selected")
        else:
            result = self.install_helm_charts(charts_to_install)
            self.return_func(refresh_installed_charts=True, returned_result=result)
        # self.text_installation_result.set_text(u'%s' % result)

    def install_helm_charts(self, charts_array):
        """
        Execute 'helm update' command on input values
        charts_array is an array of strings as:
        ['ingress-traefik', 'kubernetes-dashboard', ...]

        :param charts_array: array of strings
        :return: return code and value from execution command as dict
        """
        # Update helm repo before installation
        run(["helm", "repo", "update"], stdout=PIPE, stderr=PIPE)

        status = 0
        value = 'no errors found'
        for deployment in self.charts_dict_list:
            # Only if current chart name exists in the input 'charts_array'
            if deployment['chart_name'] in charts_array:
                # In case the deployment uses images from private repository
                # then install helm chart with setting the docker-registry secret values
                if deployment['private_image']:
                    completed_process_object = run(["helm", "upgrade", deployment['chart_name'],
                                                    "--install", deployment['helm_repo_name'],
                                                    "--namespace", deployment['name_space'],
                                                    "-f", deployment['values_file'],
                                                    "--set", "secrets.docker.registry=%s" % self.docker_registry,
                                                    "--set", "secrets.docker.username=%s" % self.docker_username,
                                                    "--set", "secrets.docker.password=%s" % self.docker_password],
                                                   stdout=PIPE,
                                                   stderr=PIPE)
                else:
                    completed_process_object = run(["helm", "upgrade", deployment['chart_name'],
                                                    "--install", deployment['helm_repo_name'],
                                                    "--namespace", deployment['name_space'],
                                                    "-f", deployment['values_file']],
                                                   stdout=PIPE,
                                                   stderr=PIPE)
                # In case of a non 0 return code, update return from last iteration
                if completed_process_object.returncode != 0:
                    status = completed_process_object.returncode
                    value = completed_process_object.stderr.decode('utf-8') + " *** Additional errors may occurred"
        return {'status': status, 'value': value}



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="helm-charts",
    version="1.0.11",
    author="Arie Lev",
    author_email="levinsonarie@gmail.com",
    description="Helm Charts installer",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ArieLevs/Helm-Charts-Installer",
    license='Apache License 2.0',
    include_package_data=True,
    packages=setuptools.find_packages(),
    install_requires=[
        'urwid',
      ],
    classifiers=(
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: Unix",
        "Operating System :: MacOS",
    ),
    entry_points={
        'console_scripts': [
            'helm_charts = helm_charts.k8s_charts_installer:main'
        ],
    },
)
